"""Base class for all objects requiring a valid names for all engines."""
from enum import Enum
from typing import Union
from pydantic import BaseModel, Field, Extra, constr


class NoExtraBaseModel(BaseModel):
    """Base class for all objects that are not extensible with additional keys."""

    class Config:
        extra = Extra.forbid


class Color(NoExtraBaseModel):
    """A RGB color."""

    type: constr(regex='^Color$') = 'Color'

    r: int = Field(
        ...,
        ge=0,
        le=255,
        description='Value for red channel.'
    )

    g: int = Field(
        ...,
        ge=0,
        le=255,
        description='Value for green channel.'
    )

    b: int = Field(
        ...,
        ge=0,
        le=255,
        description='Value for blue channel.'
    )

    a: int = Field(
        default=255,
        ge=0,
        le=255,
        description='Value for the alpha channel, which defines the opacity as a '
        'number between 0 (fully transparent) and 255 (fully opaque).'
    )


class Default(NoExtraBaseModel):
    """Object to signify when the default value of a visual interface should be used."""

    type: constr(regex='^Default$') = 'Default'


class LineTypes(str, Enum):
    continuous = 'Continuous'
    dashed = 'Dashed'
    dots = 'Dotted'
    dash_dot = 'DashDot'
    center = 'Center'
    border = 'Border'
    hidden = 'Hidden'


class DisplayModes(str, Enum):
    shaded = 'Shaded'
    surface = 'Surface'
    surface_with_edges = 'SurfaceWithEdges'
    wireframe = 'Wireframe'


class DisplayBaseModel(NoExtraBaseModel):
    """Base class for all Geometric Display objects."""

    user_data: dict = Field(
        default=None,
        description='Optional dictionary of user data associated with the object.'
        'All keys and values of this dictionary should be of a standard data '
        'type to ensure correct serialization of the object (eg. str, float, '
        'int, list).'
    )


class SingleColorBase(DisplayBaseModel):
    """Base class for all Geometric Display objects with a single color."""

    color: Color = Field(
        ...,
        description='Color for the geometry.'
    )


class LineCurveBase(SingleColorBase):
    """Base class for all Geometric Display objects with a line like properties."""

    line_width: Union[Default, float] = Field(
        Default(),
        ge=0,
        description='Number for line width in pixels (for the screen) or '
        'millimeters (in print). Set to zero to hide the geometry.'
    )

    line_type: LineTypes = Field(
        LineTypes.continuous,
        description='Text to indicate the type of line to display (dashed, dotted, '
        'etc.). The LineTypes enumeration contains all acceptable types.'
    )



"""Schemas for geometry objects in 2D space."""
import math
from typing import List
from pydantic import Field, conlist, constr, conint

from .base import NoExtraBaseModel, Color


class Vector2D(NoExtraBaseModel):
    """A vector object in 2D space."""

    type: constr(regex='^Vector2D$') = 'Vector2D'

    x: float = Field(
        ...,
        description='Number for X coordinate.'
    )

    y: float = Field(
        ...,
        description='Number for Y coordinate.'
    )


class Point2D(NoExtraBaseModel):
    """A point object in 2D space."""

    type: constr(regex='^Point2D$') = 'Point2D'

    x: float = Field(
        ...,
        description='Number for X coordinate.'
    )

    y: float = Field(
        ...,
        description='Number for Y coordinate.'
    )


class Ray2D(NoExtraBaseModel):
    """A ray object in 2D space."""

    type: constr(regex='^Ray2D$') = 'Ray2D'

    p: List[float] = Field(
        ...,
        description="Ray base point as 2 (x, y) values.",
        min_items=2,
        max_items=2
    )

    v: List[float] = Field(
        ...,
        description="Ray direction vector as 2 (x, y) values.",
        min_items=2,
        max_items=2
    )


class LineSegment2D(NoExtraBaseModel):
    """A single line segment face in 2D space."""

    type: constr(regex='^LineSegment2D$') = 'LineSegment2D'

    p: List[float] = Field(
        ...,
        description="Line segment base point as 2 (x, y) values.",
        min_items=2,
        max_items=2
    )

    v: List[float] = Field(
        ...,
        description="Line segment direction vector as 2 (x, y) values.",
        min_items=2,
        max_items=2
    )


class Polyline2D(NoExtraBaseModel):
    """A polyline in 2D space."""

    type: constr(regex='^Polyline2D$') = 'Polyline2D'

    vertices: List[conlist(float, min_items=2, max_items=2)] = Field(
        ...,
        min_items=3,
        description='A list of points representing the the vertices of the polyline. '
        'The list should include at least 3 points and each point '
        'should be a list of 2 (x, y) values.'
    )

    interpolated: bool = Field(
        False,
        description='A boolean to note whether the polyline should be interpolated '
        'between the input vertices when it is translated to other interfaces.'
    )


class Arc2D(NoExtraBaseModel):
    """A single arc or circle in 2D space."""

    type: constr(regex='^Arc2D$') = 'Arc2D'

    c: List[float] = Field(
        ...,
        description="Center of the arc as 2 (x, y) values.",
        min_items=2,
        max_items=2
    )

    r: float = Field(
        ...,
        description='A number representing the radius of the arc.'
    )

    a1: float = Field(
        default=0,
        ge=0,
        le=math.pi * 2,
        description='A number between 0 and 2 * pi for the start angle of the arc.'
    )

    a2: float = Field(
        default=math.pi * 2,
        ge=0,
        le=math.pi * 2,
        description='A number between 0 and 2 * pi for the end angle of the arc.'
    )


class Polygon2D(NoExtraBaseModel):
    """A polygon in 2D space (without holes)."""

    type: constr(regex='^Polygon2D$') = 'Polygon2D'

    vertices: List[conlist(float, min_items=2, max_items=2)] = Field(
        ...,
        min_items=3,
        description='A list of points representing the vertices of the polygon. '
        'The list should include at least 3 points and each point should be a '
        'list of 2 (x, y) values.'
    )


class Mesh2D(NoExtraBaseModel):
    """A mesh in 2D space."""

    type: constr(regex='^Mesh2D$') = 'Mesh2D'

    vertices: List[conlist(float, min_items=2, max_items=2)] = Field(
        ...,
        min_items=3,
        description='A list of points representing the vertices of the mesh. '
        'The list should include at least 3 points and each point '
        'should be a list of 2 (x, y) values.'
    )

    faces: List[conlist(conint(ge=0), min_items=3, max_items=4)] = Field(
        ...,
        min_items=1,
        description='A list of lists with each sub-list having either 3 or 4 '
        'integers. These integers correspond to indices within the list of vertices.'
    )

    colors: List[Color] = Field(
        None,
        description='An optional list of colors that correspond to either the faces '
        'of the mesh or the vertices of the mesh.'
    )




from ladybug_display_schema.display2d import DisplayVector2D, DisplayPoint2D, \
    DisplayRay2D, DisplayLineSegment2D, DisplayPolyline2D, \
    DisplayArc2D, DisplayPolygon2D, DisplayMesh2D
from ladybug_display_schema.display3d import DisplayVector3D, DisplayPoint3D, \
    DisplayRay3D, DisplayPlane, DisplayLineSegment3D, DisplayPolyline3D, \
    DisplayArc3D, DisplayFace3D, DisplayMesh3D, DisplayPolyface3D
import os

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'display')


def test_display_vector2d():
    file_path = os.path.join(target_folder, 'vector_2d.json')
    DisplayVector2D.parse_file(file_path)


def test_point2d():
    file_path = os.path.join(target_folder, 'point_2d.json')
    DisplayPoint2D.parse_file(file_path)


def test_ray2d():
    file_path = os.path.join(target_folder, 'ray_2d.json')
    DisplayRay2D.parse_file(file_path)


def test_line_segment_2d():
    file_path = os.path.join(target_folder, 'line_segment_2d.json')
    DisplayLineSegment2D.parse_file(file_path)


def test_polyline_2d():
    file_path = os.path.join(target_folder, 'polyline_2d.json')
    DisplayPolyline2D.parse_file(file_path)


def test_arc_2d():
    file_path = os.path.join(target_folder, 'arc_2d.json')
    DisplayArc2D.parse_file(file_path)


def test_polygon_2d():
    file_path = os.path.join(target_folder, 'polygon_2d.json')
    DisplayPolygon2D.parse_file(file_path)


def test_mesh_2d():
    file_path = os.path.join(target_folder, 'mesh_2d.json')
    DisplayMesh2D.parse_file(file_path)


def test_display_vector3d():
    file_path = os.path.join(target_folder, 'vector_3d.json')
    DisplayVector3D.parse_file(file_path)


def test_point3d():
    file_path = os.path.join(target_folder, 'point_3d.json')
    DisplayPoint3D.parse_file(file_path)


def test_ray3d():
    file_path = os.path.join(target_folder, 'ray_3d.json')
    DisplayRay3D.parse_file(file_path)


def test_plane():
    file_path = os.path.join(target_folder, 'plane.json')
    DisplayPlane.parse_file(file_path)


def test_line_segment_3d():
    file_path = os.path.join(target_folder, 'line_segment_3d.json')
    DisplayLineSegment3D.parse_file(file_path)


def test_polyline_3d():
    file_path = os.path.join(target_folder, 'polyline_3d.json')
    DisplayPolyline3D.parse_file(file_path)


def test_arc_3d():
    file_path = os.path.join(target_folder, 'arc_3d.json')
    DisplayArc3D.parse_file(file_path)


def test_face_3d():
    file_path = os.path.join(target_folder, 'face_3d.json')
    DisplayFace3D.parse_file(file_path)


def test_mesh_3d():
    file_path = os.path.join(target_folder, 'mesh_3d.json')
    DisplayMesh3D.parse_file(file_path)


def test_polyface_3d():
    file_path = os.path.join(target_folder, 'polyface_3d.json')
    DisplayPolyface3D.parse_file(file_path)
